@@ -260,6 +260,14 @@ oggz_add_stream (OGGZ * oggz, long serialno)
 
   ogg_stream_init (&stream->ogg_stream, (int)serialno);
 
+  stream->content = OGGZ_CONTENT_UNKNOWN;
+  stream->nr_headers = 0;
+  stream->preroll = 0;
+  stream->granulerate_n = 1;
+  stream->granulerate_d = 1;
+  stream->basegranule = 0;
+  stream->granuleshift = 0;
+
   stream->delivered_non_b_o_s = 0;
   stream->b_o_s = 1;
   stream->e_o_s = 0;
@@ -378,20 +386,6 @@ oggz_serialno_new (OGGZ * oggz)
 
 /******** OggzMetric management ********/
 
-typedef struct {
-  ogg_int64_t gr_n;
-  ogg_int64_t gr_d;
-} oggz_metric_linear_t;
-
-static ogg_int64_t
-oggz_metric_default_linear (OGGZ * oggz, long serialno, ogg_int64_t granulepos,
-			    void * user_data)
-{
-  oggz_metric_linear_t * ldata = (oggz_metric_linear_t *)user_data;
-
-  return (ldata->gr_d * granulepos / ldata->gr_n);
-}
-
 int
 oggz_set_metric_internal (OGGZ * oggz, long serialno,
 			  OggzMetric metric, void * user_data, int internal)
@@ -401,7 +395,7 @@ oggz_set_metric_internal (OGGZ * oggz, long serialno,
   if (oggz == NULL) return OGGZ_ERR_BAD_OGGZ;
 
   if (serialno == -1) {
-    if (oggz->metric_internal)
+    if (oggz->metric_internal && oggz->metric_user_data)
       oggz_free (oggz->metric_user_data);
     oggz->metric = metric;
     oggz->metric_user_data = user_data;
@@ -410,7 +404,7 @@ oggz_set_metric_internal (OGGZ * oggz, long serialno,
     stream = oggz_get_stream (oggz, serialno);
     if (stream == NULL) return OGGZ_ERR_BAD_SERIALNO;
 
-    if (stream->metric_internal)
+    if (stream->metric_internal && stream->metric_user_data)
       oggz_free (stream->metric_user_data);
     stream->metric = metric;
     stream->metric_user_data = user_data;
@@ -427,26 +421,22 @@ oggz_set_metric (OGGZ * oggz, long serialno,
   return oggz_set_metric_internal (oggz, serialno, metric, user_data, 0);
 }
 
+/*
+ * Check if a stream in an oggz has a metric
+ */
 int
-oggz_set_metric_linear (OGGZ * oggz, long serialno,
-			ogg_int64_t granule_rate_numerator,
-			ogg_int64_t granule_rate_denominator)
+oggz_stream_has_metric (OGGZ * oggz, long serialno)
 {
-  oggz_metric_linear_t * linear_data;
+  oggz_stream_t * stream;
 
-  /* we divide by the granulerate, ie. mult by gr_d/gr_n, so ensure
-   * numerator is non-zero */
-  if (granule_rate_numerator == 0) {
-    granule_rate_numerator = 1;
-    granule_rate_denominator = 0;
-  }
+  if (oggz->metric != NULL) return 1;
 
-  linear_data = oggz_malloc (sizeof (oggz_metric_linear_t));
-  linear_data->gr_n = granule_rate_numerator;
-  linear_data->gr_d = granule_rate_denominator;
+  stream = oggz_get_stream (oggz, serialno);
+  if (stream == NULL) return OGGZ_ERR_BAD_SERIALNO;
 
-  return oggz_set_metric_internal (oggz, serialno, oggz_metric_default_linear,
-				   linear_data, 1);
+  if (stream->metric != NULL) return 1;
+
+  return 0;
 }
 
 /*
